var GenericController = require( './generic' ),
    UserSchema = require( '../models/User' );

function UserController () {
  GenericController.apply( this, arguments );
}
UserController.prototype.__proto__ = GenericController.prototype;

UserController.prototype._item_id = function ( req ) {
  return req.params.user;
};

UserController.prototype._item_query = function ( req ) {
  var query = GenericController.prototype._item_query.apply( this, arguments );

  query.select( [ 'name', 'author', 'admin', 'created', 'updated' ] );

  return query;
};

UserController.prototype._index_query = function ( req ) {
  var query = GenericController.prototype._index_query.apply( this, arguments );

  query.select( [ 'name', 'author', 'admin', 'created', 'updated' ] );

  return query;
};

UserController.prototype._validation_response = function ( res, err ) {
  var self = this,
      errors = err.errors,
      message = err.message,
      code = ( message && message.substring( 0, 6 ) );

  function unknown () {
    GenericController.prototype._validation_response.call( self, res, err );
  }

  if ( code.substring( 0, 1 ) === 'E' ) {
    if ( ( code === 'E11000' ) || ( code === 'E11001' ) ) {
      this._respond( res, null, 409, 'A user with that username already exists' );
    } else {
      unknown();
    }
  } else if ( errors ) {
    if ( errors.name && ( errors.name.type === 'required' ) ) {
      this._respond( res, null, 403, 'You must provide a name' );
    } else if ( errors.password && ( errors.password.type === 'required' ) ) {
      this._respond( res, null, 403, 'You must provide a password' );
    } else {
      unknown();
    }
  } else if ( message === 'Invalid ObjectId' ) {
    this._respond( res, null, 403, 'Invalid Author ID' );
  } else {
    unknown();
  }
};

UserController.prototype._item_create = function ( req, res, cb ) {
  function item_mod ( item ) {
    item = item.toObject();
    delete item.password;
    cb( item );
  }

  GenericController.prototype._item_create.apply( this, [ req, res, item_mod ] );
};

UserController.prototype._item_update = function ( req, res, cb ) {
  function item_mod ( item ) {
    item = item.toObject();
    delete item.password;
    cb( item );
  }

  GenericController.prototype._item_update.apply( this, [ req, res, item_mod ] );
};

module.exports = UserController;

