// dependencies
var mongoose = require( 'mongoose' ),
    express = require( 'express' ),
    express_resource = require( 'express-resource' );

module.exports = {
  createServer: createServer
};

function getModels ( mongo, user_salt ) {
  var options = { mongo: mongo },
      user_options = { mongo: mongo, salt: user_salt };

  return {
    'Author': require( '../models/Author' )( options ),
    'User': require( '../models/User' )( user_options ),
    'Blog': require( '../models/Blog' )( options ),
    'Tag': require( '../models/Tag' )( options ),
    'Post': require( '../models/Post' )( options )
  };
}

function createServer ( options ) {
  var mongo = mongoose.createConnection( options.mongo ),
      models = getModels( mongo, options.salt ),
      auth_middleware = require( '../middleware/auth' )( models[ 'User' ] );

  // app configuration
  var app = express.createServer();

  app.configure( function () {
    app.use( express.logger() );
    app.use( express.bodyParser() );
    app.use( express.methodOverride() );
    app.use( auth_middleware );
    app.use( app.router );
    //app.use( express.errorHandler() );
  });

  // special path for confirming authentication
  app.get( '/auth', function ( req, res ) {
    res.send({
      uid: req.user._id
    });
  });

  // controllers
  var AuthorController = require( '../controllers/author' ),
      author_controller = new AuthorController( models[ 'Author' ] ),
      authors = app.resource( 'authors', author_controller.router() );

  var UserController = require( '../controllers/user' ),
      user_controller = new UserController( models[ 'User' ] ),
      users = app.resource( 'users', user_controller.router() );

  var BlogController = require( '../controllers/blog' ),
      blog_controller = new BlogController( models[ 'Blog' ] ),
      blogs = app.resource( 'blogs', blog_controller.router() );

  var TagController = require( '../controllers/tag' ),
      tag_controller = new TagController( models[ 'Tag' ] ),
      tags = app.resource( 'tags', tag_controller.router() );

  var PostController = require( '../controllers/post' ),
      post_controller = new PostController( models[ 'Post' ] ),
      posts = app.resource( 'posts', post_controller.router() );

  return {
    app: app,
    start: function ( cb ) {
      app.listen( options.port, cb );
    },
    stop: function () {
      app.close();
      mongo.close();
    }
  };
};

