var request = require( 'request' ),
    mongoose = require( 'mongoose' ),
    async = require( 'async' ),
    fs = require( 'fs' ),
    PATH = require( 'path' ),

    User = require( '../models/User' ),
    Author = require( '../models/Author' ),
    Blog = require( '../models/Blog' ),
    Tag = require( '../models/Tag' ),
    Post = require( '../models/Post' );

var conf_file = PATH.join( __dirname, '../conf.json' ),
    conf = JSON.parse( fs.readFileSync( conf_file ) ),
    API = 'http://localhost:' + conf.test_port,
    MONGO = conf.test_mongo;

var lib = module.exports = {

erase: function ( cb ) {
  async.parallel([
    function ( cb ) {
      User.remove( {}, cb );
    },
    function ( cb ) {
      Author.remove( {}, cb );
    },
    function ( cb ) {
      Blog.remove( {}, cb );
    },
    function ( cb ) {
      Tag.remove( {}, cb );
    },
    function ( cb ) {
      Post.remove( {}, cb );
    }
  ], cb );
},

setup: function ( cb ) {
  mongoose.connect( MONGO );

  var user = new User({
    name: 'testuser',
    password: 'password'
  });

  lib.erase( function () {
    user.save( function ( err, user ) {
      if ( err ) {
        console.log( 'ERROR', err );
        process.exit();
      } else {
        cb({
          user: user,
          password: 'password',
          teardown: lib.teardown
        });
      }
    });
  });
},

teardown: function () {
  lib.erase( function () {
    mongoose.connections[ 0 ].close();
  });
},

request: function ( user_password, options, callback ) {
  var headers = options.headers || {};

  if ( user_password ) {
    var user = user_password[ 0 ],
      password = user_password[ 1 ],
      auth = user.name + ':' + password;

    auth = new Buffer( auth, 'ascii' ).toString( 'base64' );
    auth = 'Basic ' + auth;
    headers[ 'Authorization' ] = auth;
  }

  if ( options.body ) {
    headers[ 'Content-Type' ] = 'application/json';
  }

  options.headers = headers;
  options.uri = API + options.uri;

  request( options, callback );
}

};
