var fs = require( 'fs' ),
    util = require( 'util' ),
    PATH = require( 'path' ),
    Ramen = require( 'ramen-api' );

var fn = module.exports = {

  import: function ( type, conf_path, path ) {
    var conf_file = PATH.resolve( conf_path ),
        conf = JSON.parse( fs.readFileSync( conf_file, 'utf-8' ) ),
        source = PATH.resolve( path ),
        data = JSON.parse( fs.readFileSync( source, 'utf-8' ) );

    var ramen = new Ramen({
      url: 'http://localhost:' + conf.port,
      blog: conf.blog,
      user: conf.user,
      password: conf.password
    });

    delete data.created;
    delete data.updated;

    if ( ! data._id ) {
      ramen[ type + 's' ]()
        .create( data )
        .run( function ( err, result ) {
          fn.finish( 'Created', type, data, err, result, source );
        });
    } else {
      ramen[ type + 's' ]( data._id )
        .update( data )
        .run( function ( err, result ) {
          fn.finish( 'Updated', type, data, err, result, source );
        });
    }
  },

  finish: function ( verb, type, original, err, result, source ) {
    if ( err ) {
      console.log( 'ERROR', err.code, err.description );
    } else {
      console.log( verb + ' ' + type + ' ' + result._id );
      fn.write_json( result, source );
    }
  },

  write_json: function ( json, file ) {
    var json_str = fn.print_json( json );
    fs.writeFileSync( file, json_str, 'utf-8' );
  },

  print_json: function ( obj, indent ) {
    var result = "";

    if ( ! indent ) {
      indent = "";
    }

    for ( var property in obj ) {
      var value = obj[ property ];

      if ( typeof value === 'string' ) {
        value = '"' + value.replace( /\"/g, '\\\"' ) + '"';
      } else if ( typeof value === 'object' ) {
        if ( value instanceof Array ) {
          value = fn.print_array( value );
        } else {
          value = fn.print_json( value, indent + "\t" );
        }
      }

      result += indent + '"' + property + '" : ' + value + ",\n";
    }

    result = result.substring( 0, result.length - 2 );
    result = indent + "{\n" + result + "\n" + indent + "}";

    return result;
  },

  print_array: function ( arr ) {
    var result = '[ ';
    for ( var i = 0, l = arr.length; i < l; i++ ) {
      if ( i > 0 ) {
        result += ', ';
      }
      result += '"' + arr[ i ] + '"';
    }
    return result + ' ]';
  }

};

if ( require.main === module ) {
  fn.import( process.argv[ 2 ], process.argv[ 3 ], process.argv[ 4 ] );
}

